/* "led.c" - programik do testowania rodowiska WinAVR */
/* ukad ATmega 1MHz */
/* PB0,PB1 - diody LED; PD0 - przycisk */

#define F_CPU 8000000L
#include <avr/io.h>
#include <util/delay.h>               

/*
BARGRAF:
 1   2   3   4   5
PD2 PD3 PD4 PB7 PB6
PB1 PB0 PD7 PD6 PD5
 6   7   8   9
PC2 PC3 PC4 PC5
PC0 PC1 PC2 PC3

JOYSTICK:
   PB2
PB3   PB5
   PB4
*/

#define BAR_PB_MASK 	0x03
#define BAR_PC_MASK 	0x0F
#define BAR_PD_MASK 	0xE0
#define BAR_IN1_ON		PORTB |= _BV(1);
#define BAR_IN2_ON		PORTB |= _BV(0);
#define BAR_IN3_ON		PORTD |= _BV(7);
#define BAR_IN4_ON		PORTD |= _BV(6);
#define BAR_IN5_ON		PORTD |= _BV(5);
#define BAR_IN1_OFF		PORTB &= ~_BV(1);
#define BAR_IN2_OFF		PORTB &= ~_BV(0);
#define BAR_IN3_OFF		PORTD &= ~_BV(7);
#define BAR_IN4_OFF		PORTD &= ~_BV(6);
#define BAR_IN5_OFF		PORTD &= ~_BV(5);
#define BAR_OUT1_ON		PORTC |= _BV(0);
#define BAR_OUT2_ON		PORTC |= _BV(1);
#define BAR_OUT3_ON		PORTC |= _BV(2);
#define BAR_OUT4_ON		PORTC |= _BV(3);
#define BAR_OUT1_OFF	PORTC &= ~_BV(0);
#define BAR_OUT2_OFF	PORTC &= ~_BV(1);
#define BAR_OUT3_OFF	PORTC &= ~_BV(2);
#define BAR_OUT4_OFF	PORTC &= ~_BV(3);
#define BAR_WAIT		_delay_us(500);
#define BAR_RED			0
#define BAR_GRN			1

char bargraf[10][2];
int bargraf_blinknum = 0;
int bargraf_blink = 0;

void bargraf_init();
void bargraf_draw();

void bargraf_init(){

	DDRB |= BAR_PB_MASK;	// 1100 0000
	DDRC |= BAR_PC_MASK;	// 0011 1100
	DDRD |= BAR_PD_MASK;	// 0001 1100

	BAR_IN1_OFF
	BAR_IN2_OFF
	BAR_IN3_OFF
	BAR_IN4_OFF
	BAR_IN5_OFF
	
	BAR_OUT1_ON
	BAR_OUT2_ON
	BAR_OUT3_ON
	BAR_OUT4_ON	
	
	for(int i=0;i<10;i++)
	  for(int j=0;j<2;j++)
	    bargraf[i][j]=0;

}

void bargraf_draw(){

	BAR_IN1_OFF
	BAR_IN2_OFF
	BAR_IN3_OFF
	BAR_IN4_OFF
	BAR_IN5_OFF
	bargraf_blinknum++;
	if(bargraf_blinknum>=50){ bargraf_blink = (bargraf_blink==0?1:0); bargraf_blinknum=0;}
	
	for(int i=0;i<10;i+=2){
	       if(i>=8) BAR_IN5_ON
	  else if(i>=6) BAR_IN4_ON
	  else if(i>=4) BAR_IN3_ON
	  else if(i>=2) BAR_IN2_ON
	  else if(i>=0) BAR_IN1_ON
	  
	  if(bargraf[i][BAR_GRN]==1 || (bargraf[i][BAR_GRN]==2 && bargraf_blink==1)){
	    BAR_OUT1_OFF
		BAR_WAIT
		BAR_OUT1_ON
	  }
	  else BAR_WAIT
	  if(bargraf[i][BAR_RED]==1 || (bargraf[i][BAR_RED]==2 && bargraf_blink==0)){
	    BAR_OUT2_OFF
		BAR_WAIT
		BAR_OUT2_ON
	  }
	  else BAR_WAIT
	  if(bargraf[i+1][BAR_GRN]==1 || (bargraf[i+1][BAR_GRN]==2 && bargraf_blink==1)){
	    BAR_OUT3_OFF
		BAR_WAIT
		BAR_OUT3_ON
	  }
	  else BAR_WAIT
	  if(bargraf[i+1][BAR_RED]==1 || (bargraf[i+1][BAR_RED]==2 && bargraf_blink==0)){
	    BAR_OUT4_OFF
		BAR_WAIT
		BAR_OUT4_ON
	  }
	  else BAR_WAIT
	  
	       if(i>=8) BAR_IN5_OFF
	  else if(i>=6) BAR_IN4_OFF
	  else if(i>=4) BAR_IN3_OFF
	  else if(i>=2) BAR_IN2_OFF
	  else if(i>=0) BAR_IN1_OFF
	
	}

}

void bargraf_pasek(int kolor, int ile,int status){
	int i;
    for(int i=0; i<10-ile;i++)		bargraf[i][kolor]=0;
	for(int i=10-ile; i<10;i++)	bargraf[i][kolor]=status;
}

void bargraf_clear(){
	int i;
	for(i=0;i<10;i++){
		bargraf[i][BAR_GRN]=0;
		bargraf[i][BAR_RED]=0;
	}
}

int main(void)
{
	//Zmienne
	int minuty=5;
	int minuty2=5;
	int i;
	int num=0;
	int pulse=0;
	int progress=5;

	//Dioda zasilania
    DDRC  |=  _BV(4);
	PORTC &=  ~_BV(4);

	//Dioda aktywnoci
    DDRC  |=  _BV(5);
	PORTC &=  ~_BV(5);
	
	//Przyciski
	DDRD  &= ~_BV(2) | ~_BV(3) | ~_BV(4);
	PORTD |=  _BV(2) |  _BV(3) |  _BV(4);
	//UP-2, CENTER-3, DOWN-4??
	
	//Speaker
	//DDRC |= _BV(1);
	//PORTC &= ~_BV(1);
	
	//Inicjalizacja
	bargraf_init();
	bargraf_pasek(BAR_GRN,progress,1);

	while(1){
		if(!(PIND & _BV(2)) && progress<10){
			progress++;
			bargraf_pasek(BAR_GRN,progress,1);
			while(!(PIND & _BV(2)))bargraf_draw();
		}
		if(!(PIND & _BV(3)) && progress>0){
			progress=5;
			bargraf_pasek(BAR_GRN,progress,1);
			while(!(PIND & _BV(3)))bargraf_draw();
		}
		if(!(PIND & _BV(4)) && progress>0){
			progress--;
			bargraf_pasek(BAR_GRN,progress,1);
			while(!(PIND & _BV(4)))bargraf_draw();
		}
		pulse++;
		if(pulse == 45){ PORTC |=  _BV(4); }
		if(pulse >= 50){ PORTC &= ~_BV(4); pulse=0; }
		//if(num++ >= 500){ num=0; PORTC^=_BV(5); }
		bargraf_draw();
	}
}
